from enum import Enum


class InvoicePartyEnum(str, Enum):
    yoo_money = "yoo_money"
    merchant = "merchant"


class InvoiceReasonEnum(str, Enum):
    invoice_canceled = "invoice_canceled"
    invoice_expired = "invoice_expired"
    general_decline = "general_decline"
    payment_canceled = "payment_canceled"
    payment_expired_on_capture = "payment_expired_on_capture"



import uuid
from typing import Any

from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.refund_request import RefundRequest
from async_yookassa.models.refund_response import RefundListResponse, RefundResponse
from async_yookassa.utils import get_base_headers


class Refund:
    """
    Класс, представляющий модель Refund.
    """

    base_path = "/refunds"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def create(cls, params: dict[str, Any] | RefundRequest, idempotency_key: uuid.UUID | None = None):
        """
        Создание возврата

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа RefundResponse, возвращаемого API при запросе информации о возврате
        """
        instance = cls()

        path = cls.base_path

        headers = get_base_headers(idempotency_key=idempotency_key)

        if isinstance(params, dict):
            params_object = RefundRequest(**params)
        elif isinstance(params, RefundRequest):
            params_object = params
        else:
            raise TypeError("Invalid params value type")

        response = await instance.client.request(
            body=params_object, method=HTTPMethodEnum.POST, path=path, headers=headers
        )

        return RefundResponse(**response)

    @classmethod
    async def find_one(cls, refund_id):
        """
        Возвращает информацию о возврате

        :param refund_id: Уникальный идентификатор возврата
        :return: Объект ответа RefundResponse, возвращаемого API при запросе информации о возврате
        """
        instance = cls()

        if not isinstance(refund_id, str):
            raise ValueError("Invalid payment_id value")

        path = instance.base_path + "/" + refund_id

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path)

        return RefundResponse(**response)

    @classmethod
    async def list(cls, params):
        """
        Возвращает список возвратов

        :param params: Данные передаваемые в API
        :return: Объект ответа RefundListResponse, возвращаемого API при запросе списка возвратов
        """
        instance = cls()

        path = cls.base_path

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path, query_params=params)

        return RefundListResponse(**response)



import re
from decimal import Decimal
from typing import Any

from pydantic import BaseModel, Field, field_validator

from async_yookassa.models.payment_submodels.amount import Amount
from async_yookassa.models.payment_submodels.receipt_submodels.mark_code_info import (
    MarkCodeInfo,
)
from async_yookassa.models.payment_submodels.receipt_submodels.mark_quantity import (
    MarkQuantity,
)
from async_yookassa.models.payment_submodels.receipt_submodels.payment_subject_industry_details import (
    PaymentSubjectIndustryDetails,
)


class ReceiptItem(BaseModel):
    description: str = Field(max_length=128)
    amount: Amount
    vat_code: int = Field(le=6)
    quantity: str
    measure: str | None = None
    mark_quantity: MarkQuantity | None = None
    payment_subject: str | None = None
    payment_mode: str | None = None
    country_of_origin_code: str | None = Field(min_length=2, max_length=2, default=None)
    customs_declaration_number: str | None = Field(max_length=32, default=None)
    excise: str | None = None
    product_code: str | None = None
    mark_code_info: MarkCodeInfo | None = None
    mark_mode: str | None = None
    payment_subject_industry_details: PaymentSubjectIndustryDetails | None = None

    @field_validator("quantity", mode="before")
    def quantity_validator(cls, value: Any) -> str:
        """
        Устанавливает quantity модели ReceiptItem.

        :param value: quantity модели ReceiptItem.
        :type value: Decimal
        """
        return str(Decimal(str(float(value))))

    @field_validator("excise", mode="before")
    def excise_validator(cls, value: Any) -> str:
        """ "
        Устанавливает excise модели ReceiptItem.

        :param value: excise модели ReceiptItem.
        :type value: str
        """
        return str(Decimal(str(value)))

    @field_validator("mark_mode", mode="before")
    def mark_mode_validator(cls, value: str) -> str:
        """
        Устанавливает mark_mode модели ReceiptItem.

        :param value: mark_mode модели ReceiptItem.
        :type value: str
        """
        if not re.search(r"^0$", value):
            raise ValueError(r"Invalid value for `mark_mode`, must be a follow pattern or equal to `/^0$/`")

        return value



from async_yookassa.exceptions.api_error import APIError


class UnauthorizedError(APIError):
    """
    [Basic Auth] Неверный идентификатор вашего аккаунта в ЮKassa или секретный ключ
    (имя пользователя и пароль при аутентификации).
    [OAuth 2.0] Невалидный OAuth-токен: он некорректный, устарел или его отозвали. Запросите токен заново.
    """

    HTTP_CODE = 401



import re
from datetime import datetime

from pydantic import BaseModel, Field, field_validator


class Leg(BaseModel):
    departure_airport: str = Field(min_length=3, max_length=3)
    destination_airport: str = Field(min_length=3, max_length=3)
    departure_date: str
    carrier_code: str | None = Field(min_length=2, max_length=3, default=None)

    @field_validator("departure_airport", mode="before")
    def departure_airport_validator(cls, value: str) -> str:
        """
        Устанавливает departure_airport модели Leg.

        :param value: departure_airport модели Leg.
        :type value: str
        """
        if not re.match("^[A-Z]{3}$", value):
            raise ValueError(
                r"Invalid value for `departure_airport`, must be a follow pattern or equal to `/^[A-Z]{3}/`"
            )

        return value

    @field_validator("destination_airport", mode="before")
    def destination_airport_validator(cls, value: str) -> str:
        """
        Устанавливает destination_airport модели Leg.

        :param value: destination_airport модели Leg.
        :type value: str
        """
        if not re.match("^[A-Z]{3}$", value):
            raise ValueError(
                r"Invalid value for `destination_airport`, must be a follow pattern or equal to `/^[A-Z]{3}/`"
            )

        return value

    @field_validator("departure_date", mode="before")
    def departure_date_validator(cls, value: str) -> str:
        """
        Устанавливает departure_date модели Leg.

        :param value: departure_date модели Leg.
        :type value: datetime
        """
        try:
            datetime.strptime(value, "%Y-%m-%d")
        except ValueError:
            raise ValueError("Incorrect data format, should be YYYY-MM-DD")

        return value



from async_yookassa.configuration import Configuration
from async_yookassa.invoice import Invoice
from async_yookassa.payment import Payment
from async_yookassa.refund import Refund

__author__ = "Ivan Ashikhmin and YooMoney"
__email__ = "sushkoos@gmail.com and cms@yoomoney.ru"
__version__ = "0.2.5"
__all__ = ["Configuration", "Payment", "Invoice", "Refund"]



from dataclasses import dataclass
from typing import Self

from async_yookassa.exceptions.configuration_errors import ConfigurationError
from async_yookassa.models.configuration_submodels.version import Version


@dataclass
class Configuration:
    api_url: str = "https://api.yookassa.ru/v3"
    account_id: str | None = None
    secret_key: str | None = None
    timeout: int = 1800
    max_attempts: int = 3
    auth_token: str | None = None
    agent_framework: Version | None = None
    agent_cms: Version | None = None
    agent_module: Version | None = None

    def __post_init__(self):
        self.assert_has_api_credentials()

    @classmethod
    def configure(cls, account_id: str, secret_key: str, **kwargs) -> None:
        """
        Устанавливает настройки конфигурации для базовой авторизации.

        :param account_id: Идентификатор магазина.
        :param secret_key: Секретный ключ.
        :param kwargs: Словарь с дополнительными параметрами.
        """
        cls.account_id = account_id
        cls.secret_key = secret_key
        cls.auth_token = None
        cls.api_url = kwargs.get("api_url", cls.api_url)
        cls.timeout = kwargs.get("timeout", cls.timeout)
        cls.max_attempts = kwargs.get("max_attempts", cls.max_attempts)

    @classmethod
    def configure_auth_token(cls, token: str, **kwargs) -> None:
        """
        Устанавливает настройки конфигурации для авторизации по OAuth.

        :param token: Ключ OAuth.
        :param kwargs: Словарь с дополнительными параметрами.
        """
        cls.account_id = None
        cls.secret_key = None
        cls.auth_token = token
        cls.api_url = kwargs.get("api_url", cls.api_url)
        cls.timeout = kwargs.get("timeout", cls.timeout)
        cls.max_attempts = kwargs.get("max_attempts", cls.max_attempts)

    def configure_user_agent(
        self, framework: Version | None = None, cms: Version | None = None, module: Version | None = None
    ) -> None:
        """
        Устанавливает настройки конфигурации для User-Agent.

        :param framework: Версия фреймворка.
        :param cms: Версия CMS.
        :param module: Версия модуля.
        """
        if isinstance(framework, Version):
            self.agent_framework = framework
        if isinstance(cms, Version):
            self.agent_cms = cms
        if isinstance(module, Version):
            self.agent_module = module

    @classmethod
    def instantiate(cls) -> Self:
        """
        Получение объекта конфигурации.

        :return: Объект конфигурации
        """
        return cls(
            account_id=cls.account_id,
            secret_key=cls.secret_key,
            timeout=cls.timeout,
            max_attempts=cls.max_attempts,
            auth_token=cls.auth_token,
            agent_framework=cls.agent_framework,
            agent_cms=cls.agent_cms,
            agent_module=cls.agent_module,
            api_url=cls.api_url,
        )

    @staticmethod
    def api_endpoint() -> str:
        """Возвращает URL для API Кассы."""
        return Configuration.api_url

    def has_api_credentials(self) -> bool:
        """Проверка наличия параметров базовой авторизации."""
        return self.account_id is not None and self.secret_key is not None

    def assert_has_api_credentials(self) -> None:
        """Проверка наличия API параметров, выброс исключения ConfigurationError при ошибке."""
        if self.auth_token is None and not self.has_api_credentials():
            raise ConfigurationError("account_id and secret_key are required")
        elif self.auth_token and self.has_api_credentials():
            raise ConfigurationError("Could not configure authorization with both auth_token and basic auth")



from datetime import datetime

from pydantic import BaseModel, Field

from async_yookassa.enums.payment_response import ReceiptRegistrationEnum
from async_yookassa.models.payment_submodels.amount import Amount
from async_yookassa.models.payment_submodels.cancellation_details import RefundDetails
from async_yookassa.models.payment_submodels.deal import DealRefund
from async_yookassa.models.payment_submodels.payment_method import PaymentMethodRefund
from async_yookassa.models.payment_submodels.transfers import TransferBase


class RefundResponse(BaseModel):
    id: str = Field(min_length=36, max_length=36)
    payment_id: str = Field(min_length=36, max_length=36)
    status: ReceiptRegistrationEnum
    cancellation_details: RefundDetails | None = None
    receipt_registration: ReceiptRegistrationEnum | None = None
    created_at: datetime
    amount: Amount
    description: str | None = Field(max_length=250, default=None)
    sources: list[TransferBase] | None = None
    deal: DealRefund | None = None
    refund_method: PaymentMethodRefund | None = None


class RefundListResponse(BaseModel):
    type: str
    items: list[RefundResponse]
    next_cursor: str | None = None



from pydantic import BaseModel, Field


class MarkCodeInfo(BaseModel):
    mark_code_raw: str | None = None
    unknown: str | None = Field(min_length=1, max_length=32, default=None)
    ean_8: str | None = Field(min_length=8, max_length=8, default=None)
    ean_13: str | None = Field(min_length=13, max_length=13, default=None)
    itf_14: str | None = Field(min_length=14, max_length=14, default=None)
    gs_10: str | None = Field(min_length=1, max_length=38, default=None)
    gs_1m: str | None = Field(min_length=1, max_length=200, default=None)
    short: str | None = Field(min_length=1, max_length=38, default=None)
    fur: str | None = Field(min_length=20, max_length=20, default=None)
    egais_20: str | None = Field(min_length=33, max_length=33, default=None)
    egais_30: str | None = Field(min_length=14, max_length=14, default=None)



import re

from pydantic import BaseModel, Field, field_validator


class PaymentSubjectIndustryDetails(BaseModel):
    federal_id: str
    document_date: str
    document_number: str = Field(max_length=32)
    value: str = Field(max_length=256)

    @field_validator("federal_id", mode="before")
    def federal_id_validator(cls, value: str) -> str:
        """
        Устанавливает federal_id модели PaymentSubjectIndustryDetails.

        :param value: federal_id модели PaymentSubjectIndustryDetails.
        :type value: str
        """
        if not re.search(r"(^00[1-9]$)|(^0[1-6][0-9]$)|(^07[0-3]$)", value):
            raise ValueError(
                r"Invalid value for `federal_id`, "
                r"must be a follow pattern or equal to `/(^00[1-9]$)|(^0[1-6]{1}[0-9]$)|(^07[0-3]$)/`"
            )

        return value



from decimal import ROUND_HALF_UP, Decimal
from typing import Any

from pydantic import BaseModel, field_validator


class Amount(BaseModel):
    value: str
    currency: str

    @field_validator("value", mode="before")
    def validate_value(cls, value: Any) -> str:
        """
        Устанавливает value модели Amount.

        :param value: value модели Amount.
        :type value: Decimal
        """
        return str(Decimal(str(float(value))).quantize(Decimal("1.11"), rounding=ROUND_HALF_UP))

    @field_validator("currency", mode="before")
    def validate_currency(cls, value: str) -> str:
        """
        Устанавливает currency модели Amount.

        :param value: currency модели Amount.
        :type value: str
        """
        if not isinstance(value, str):
            raise ValueError("currency must be a string")

        if len(value) != 3:
            raise ValueError("currency must be 3 characters long")

        return value.upper()



from pydantic import BaseModel, ConfigDict

from async_yookassa.enums.settlement import SettlementTypeEnum
from async_yookassa.models.payment_submodels.amount import Amount


class Settlement(BaseModel):
    type: str = SettlementTypeEnum.payout
    amount: Amount

    model_config = ConfigDict(use_enum_values=True)



from pydantic import BaseModel, ConfigDict, Field, model_validator

from async_yookassa.enums.confirmation import ConfirmationTypeEnum


class ConfirmationBase(BaseModel):
    type: ConfirmationTypeEnum
    return_url: str | None = Field(max_length=2048, default=None)
    enforce: bool | None = None


class Confirmation(ConfirmationBase):
    locale: str | None = None

    model_config = ConfigDict(use_enum_values=True)

    @model_validator(mode="before")
    def validate_required_fields(cls, values):
        type_value = values.get("type")

        if type_value == ConfirmationTypeEnum.mobile_application:
            if not values.get("return_url"):
                raise ValueError("Field 'return_url' is required for type 'mobile_application'")

        if type_value == ConfirmationTypeEnum.redirect:
            if not values.get("return_url"):
                raise ValueError("Field 'return_url' is required for type 'redirect'")

        return values


class ConfirmationResponse(ConfirmationBase):
    confirmation_token: str | None = None
    confirmation_url: str | None = None
    confirmation_data: str | None = None

    @model_validator(mode="before")
    def validate_required_fields(cls, values):
        type_value = values.get("type")

        if type_value == ConfirmationTypeEnum.embedded:
            if not values.get("confirmation_token"):
                raise ValueError("Field 'confirmation_token' is required for type 'embedded'")
        elif type_value == ConfirmationTypeEnum.mobile_application:
            if not values.get("confirmation_url"):
                raise ValueError("Field 'confirmation_url' is required for type 'mobile_application'")
        elif type_value == ConfirmationTypeEnum.redirect:
            if not values.get("confirmation_url"):
                raise ValueError("Field 'confirmation_url' is required for type 'redirect'")
        elif type_value == ConfirmationTypeEnum.qr:
            if not values.get("confirmation_data"):
                raise ValueError("Field 'confirmation_data' is required for type 'qr'")

        return values



import uuid


def get_base_headers(idempotency_key: uuid.UUID | None = None) -> dict[str, str]:
    if not idempotency_key:
        idempotency_key = uuid.uuid4()
    return {"Idempotence-Key": str(idempotency_key)}



from pydantic import BaseModel, Field, model_validator

from async_yookassa.enums.receiver import ReceiverTypeEnum


class Receiver(BaseModel):
    type: ReceiverTypeEnum
    account_number: str | None = Field(min_length=20, max_length=20, default=None)
    bic: str | None = Field(min_length=9, max_length=9, default=None)
    phone: str | None = Field(min_length=11, max_length=15, default=None)

    @model_validator(mode="before")
    def validate_required_fields(cls, values):
        type_value = values.get("type")

        if type_value == ReceiverTypeEnum.bank_account:
            if not values.get("account_number"):
                raise ValueError("Field 'account_number' is required for type 'bank_account'")
            if not values.get("bic"):
                raise ValueError("Field 'bic' is required for type 'bank_account'")

        elif type_value == ReceiverTypeEnum.mobile_balance:
            if not values.get("phone"):
                raise ValueError("Field 'phone' are required for type 'mobile_balance'")

        elif type_value == ReceiverTypeEnum.digital_wallet:
            if not values.get("account_number"):
                raise ValueError("Field 'account_number' are required for type 'digital_wallet'")

        return values



from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.amount import Amount
from async_yookassa.models.payment_submodels.deal import DealRefund
from async_yookassa.models.payment_submodels.payment_method import PaymentMethodRefund
from async_yookassa.models.payment_submodels.receipt import Receipt
from async_yookassa.models.payment_submodels.transfers import TransferBase


class RefundRequest(BaseModel):
    payment_id: str = Field(min_length=36, max_length=36)
    amount: Amount
    description: str | None = Field(max_length=250, default=None)
    receipt: Receipt | None = None
    sources: list[TransferBase] | None = None
    deal: DealRefund | None = None
    refund_method_data: PaymentMethodRefund | None = None



from enum import Enum


class VatDataTypeEnum(str, Enum):
    untaxed = "untaxed"
    calculated = "calculated"
    mixed = "mixed"


class RateEnum(str, Enum):
    seven = "7"
    ten = "10"
    eighteen = "18"
    twenty = "20"



import uuid
from typing import Any

from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.invoice_request import InvoiceRequest
from async_yookassa.models.invoice_response import InvoiceResponse
from async_yookassa.utils import get_base_headers


class Invoice:
    """
    Класс, представляющий модель Invoice.
    """

    base_path = "/invoices"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def find_one(cls, invoice_id: str):
        """
        Возвращает информацию о счёте

        :param invoice_id: Уникальный идентификатор счёта
        :return: Объект ответа InvoiceResponse, возвращаемого API при запросе счёта
        """
        instance = cls()

        if not isinstance(invoice_id, str):
            raise ValueError("Invalid invoice_id value")

        path = instance.base_path + "/" + invoice_id

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path)
        return InvoiceResponse(**response)

    @classmethod
    async def create(cls, params: dict[str, Any] | InvoiceRequest, idempotency_key: uuid.UUID | None = None):
        """
        Создание счёта

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: InvoiceResponse Объект ответа, возвращаемого API при запросе выплате
        """
        instance = cls()

        path = cls.base_path

        headers = get_base_headers(idempotency_key=idempotency_key)

        if isinstance(params, dict):
            params_object = InvoiceRequest(**params)
        elif isinstance(params, InvoiceRequest):
            params_object = params
        else:
            raise TypeError("Invalid params value type")

        response = await instance.client.request(
            body=params_object, method=HTTPMethodEnum.POST, path=path, headers=headers
        )
        return InvoiceResponse(**response)



from enum import Enum


class PaymentMethodTypeEnum(str, Enum):
    sber_loan = "sber_loan"
    alfabank = "alfabank"
    mobile_balance = "mobile_balance"
    bank_card = "bank_card"
    installments = "installments"
    cash = "cash"
    sbp = "sbp"
    b2b_sberbank = "b2b_sberbank"
    electronic_certificate = "electronic_certificate"
    yoo_money = "yoo_money"
    apple_pay = "apple_pay"
    google_pay = "google_pay"
    qiwi = "qiwi"
    sberbank = "sberbank"
    tinkoff_bank = "tinkoff_bank"
    wechat = "wechat"
    webmoney = "webmoney"
